// Copyright (c) Microsoft Corporation
// All rights reserved. 
//
// Licensed under the Apache License, Version 2.0 (the ""License""); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 
//
// THIS CODE IS PROVIDED ON AN  *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT. 
//
// See the Apache Version 2.0 License for specific language governing permissions and limitations under the License.

#define NTDDI_VERSION 0x06010000

#include <v8.h>
#include <string>
#include <node_object_wrap.h>
#include <ppltasks.h>
#include "CollectionsConverter.h"
#include "CollectionsWrap.h"
#include "node-async.h"
#include "NodeRtUtils.h"
#include "OpaqueWrapper.h"
#include "WrapperBase.h"

#using <Windows.WinMD>

// this undefs fixes the issues of compiling Windows.Data.Json, Windows.Storag.FileProperties, and Windows.Stroage.Search
// Some of the node header files brings windows definitions with the same names as some of the WinRT methods
#undef DocumentProperties
#undef GetObject
#undef CreateEvent
#undef FindText
#undef SendMessage

const char* REGISTRATION_TOKEN_MAP_PROPERTY_NAME = "__registrationTokenMap__";

using namespace v8;
using namespace node;
using namespace concurrency;

namespace NodeRT { namespace Windows { namespace Devices { namespace Scanners { 
  v8::Handle<v8::Value> WrapIImageScannerFormatConfiguration(::Windows::Devices::Scanners::IImageScannerFormatConfiguration^ wintRtInstance);
  ::Windows::Devices::Scanners::IImageScannerFormatConfiguration^ UnwrapIImageScannerFormatConfiguration(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapImageScannerAutoConfiguration(::Windows::Devices::Scanners::ImageScannerAutoConfiguration^ wintRtInstance);
  ::Windows::Devices::Scanners::ImageScannerAutoConfiguration^ UnwrapImageScannerAutoConfiguration(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapIImageScannerSourceConfiguration(::Windows::Devices::Scanners::IImageScannerSourceConfiguration^ wintRtInstance);
  ::Windows::Devices::Scanners::IImageScannerSourceConfiguration^ UnwrapIImageScannerSourceConfiguration(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapImageScannerFlatbedConfiguration(::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^ wintRtInstance);
  ::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^ UnwrapImageScannerFlatbedConfiguration(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapImageScannerFeederConfiguration(::Windows::Devices::Scanners::ImageScannerFeederConfiguration^ wintRtInstance);
  ::Windows::Devices::Scanners::ImageScannerFeederConfiguration^ UnwrapImageScannerFeederConfiguration(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapImageScannerScanResult(::Windows::Devices::Scanners::ImageScannerScanResult^ wintRtInstance);
  ::Windows::Devices::Scanners::ImageScannerScanResult^ UnwrapImageScannerScanResult(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapImageScannerPreviewResult(::Windows::Devices::Scanners::ImageScannerPreviewResult^ wintRtInstance);
  ::Windows::Devices::Scanners::ImageScannerPreviewResult^ UnwrapImageScannerPreviewResult(Handle<Value> value);
  
  v8::Handle<v8::Value> WrapImageScanner(::Windows::Devices::Scanners::ImageScanner^ wintRtInstance);
  ::Windows::Devices::Scanners::ImageScanner^ UnwrapImageScanner(Handle<Value> value);
  


  static v8::Handle<v8::Value> InitImageScannerFormatEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("ImageScannerFormat"), enumObject);

    enumObject->Set(String::NewSymbol("jpeg"), Integer::New(0));
    enumObject->Set(String::NewSymbol("png"), Integer::New(1));
    enumObject->Set(String::NewSymbol("deviceIndependentBitmap"), Integer::New(2));
    enumObject->Set(String::NewSymbol("tiff"), Integer::New(3));
    enumObject->Set(String::NewSymbol("xps"), Integer::New(4));
    enumObject->Set(String::NewSymbol("openXps"), Integer::New(5));
    enumObject->Set(String::NewSymbol("pdf"), Integer::New(6));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitImageScannerAutoCroppingModeEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("ImageScannerAutoCroppingMode"), enumObject);

    enumObject->Set(String::NewSymbol("disabled"), Integer::New(0));
    enumObject->Set(String::NewSymbol("singleRegion"), Integer::New(1));
    enumObject->Set(String::NewSymbol("multipleRegion"), Integer::New(2));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitImageScannerColorModeEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("ImageScannerColorMode"), enumObject);

    enumObject->Set(String::NewSymbol("color"), Integer::New(0));
    enumObject->Set(String::NewSymbol("grayscale"), Integer::New(1));
    enumObject->Set(String::NewSymbol("monochrome"), Integer::New(2));
    enumObject->Set(String::NewSymbol("autoColor"), Integer::New(3));

    return scope.Close(Undefined());
  }


  static v8::Handle<v8::Value> InitImageScannerScanSourceEnum(const Handle<Object> exports)
  {
    HandleScope scope;
    
    Handle<Object> enumObject = Object::New();
    exports->Set(String::NewSymbol("ImageScannerScanSource"), enumObject);

    enumObject->Set(String::NewSymbol("default"), Integer::New(0));
    enumObject->Set(String::NewSymbol("flatbed"), Integer::New(1));
    enumObject->Set(String::NewSymbol("feeder"), Integer::New(2));
    enumObject->Set(String::NewSymbol("autoConfigured"), Integer::New(3));

    return scope.Close(Undefined());
  }



  
  static bool IsImageScannerResolutionJsObject(Handle<Value> value)
  {
    if (!value->IsObject())
    {
      return false;
    }

    Handle<String> symbol;
    Handle<Object> obj = value.As<Object>();

    symbol = String::NewSymbol("dpiX");
    if (obj->Has(symbol))
    {
      if (!obj->Get(symbol)->IsNumber())
      {
          return false;
      }
    }
    
    symbol = String::NewSymbol("dpiY");
    if (obj->Has(symbol))
    {
      if (!obj->Get(symbol)->IsNumber())
      {
          return false;
      }
    }
    
    return true;
  }

  ::Windows::Devices::Scanners::ImageScannerResolution ImageScannerResolutionFromJsObject(Handle<Value> value)
  {
    HandleScope scope;
    ::Windows::Devices::Scanners::ImageScannerResolution returnValue;
    
    if (!value->IsObject())
    {
      ThrowException(Exception::TypeError(NodeRT::Utils::NewString(L"Unexpected type, expected an object")));
      return returnValue;
    }

    Handle<Object> obj = value.As<Object>();
    Handle<String> symbol;

    symbol = String::NewSymbol("dpiX");
    if (obj->Has(symbol))
    {
      returnValue.DpiX = static_cast<float>(obj->Get(symbol)->NumberValue());
    }
    
    symbol = String::NewSymbol("dpiY");
    if (obj->Has(symbol))
    {
      returnValue.DpiY = static_cast<float>(obj->Get(symbol)->NumberValue());
    }
    
    return returnValue;
  }

  Handle<Value> ImageScannerResolutionToJsObject(::Windows::Devices::Scanners::ImageScannerResolution value)
  {
    HandleScope scope;

    Handle<Object> obj = Object::New();

    obj->Set(String::NewSymbol("dpiX"), Number::New(static_cast<double>(value.DpiX)));
    obj->Set(String::NewSymbol("dpiY"), Number::New(static_cast<double>(value.DpiY)));
    
    return scope.Close(obj);
  }

  
  static bool IsSizeJsObject(Handle<Value> value)
  {
    if (!value->IsObject())
    {
      return false;
    }

    Handle<String> symbol;
    Handle<Object> obj = value.As<Object>();

    return true;
  }

  ::Windows::Foundation::Size SizeFromJsObject(Handle<Value> value)
  {
    HandleScope scope;
    ::Windows::Foundation::Size returnValue;
    
    if (!value->IsObject())
    {
      ThrowException(Exception::TypeError(NodeRT::Utils::NewString(L"Unexpected type, expected an object")));
      return returnValue;
    }

    Handle<Object> obj = value.As<Object>();
    Handle<String> symbol;

    return returnValue;
  }

  Handle<Value> SizeToJsObject(::Windows::Foundation::Size value)
  {
    HandleScope scope;

    Handle<Object> obj = Object::New();

    
    return scope.Close(obj);
  }

  
  static bool IsRectJsObject(Handle<Value> value)
  {
    if (!value->IsObject())
    {
      return false;
    }

    Handle<String> symbol;
    Handle<Object> obj = value.As<Object>();

    return true;
  }

  ::Windows::Foundation::Rect RectFromJsObject(Handle<Value> value)
  {
    HandleScope scope;
    ::Windows::Foundation::Rect returnValue;
    
    if (!value->IsObject())
    {
      ThrowException(Exception::TypeError(NodeRT::Utils::NewString(L"Unexpected type, expected an object")));
      return returnValue;
    }

    Handle<Object> obj = value.As<Object>();
    Handle<String> symbol;

    return returnValue;
  }

  Handle<Value> RectToJsObject(::Windows::Foundation::Rect value)
  {
    HandleScope scope;

    Handle<Object> obj = Object::New();

    
    return scope.Close(obj);
  }

  
  class IImageScannerFormatConfiguration : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("IImageScannerFormatConfiguration"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("isFormatSupported"), FunctionTemplate::New(IsFormatSupported)->GetFunction());
      
                        
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("defaultFormat"), DefaultFormatGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("format"), FormatGetter, FormatSetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("IImageScannerFormatConfiguration"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    IImageScannerFormatConfiguration(::Windows::Devices::Scanners::IImageScannerFormatConfiguration^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Devices::Scanners::IImageScannerFormatConfiguration^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerFormatConfiguration^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Devices::Scanners::IImageScannerFormatConfiguration^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      IImageScannerFormatConfiguration *wrapperInstance = new IImageScannerFormatConfiguration(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> IsFormatSupported(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerFormatConfiguration^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerFormatConfiguration *wrapper = IImageScannerFormatConfiguration::Unwrap<IImageScannerFormatConfiguration>(args.This());

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Devices::Scanners::ImageScannerFormat arg0 = static_cast<::Windows::Devices::Scanners::ImageScannerFormat>(args[0]->Int32Value());
          
          bool result;
          result = wrapper->_instance->IsFormatSupported(arg0);
          return scope.Close(Boolean::New(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



    static Handle<Value> DefaultFormatGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerFormatConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerFormatConfiguration *wrapper = IImageScannerFormatConfiguration::Unwrap<IImageScannerFormatConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerFormat result = wrapper->_instance->DefaultFormat;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> FormatGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerFormatConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerFormatConfiguration *wrapper = IImageScannerFormatConfiguration::Unwrap<IImageScannerFormatConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerFormat result = wrapper->_instance->Format;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void FormatSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerFormatConfiguration^>(info.This()))
      {
        return;
      }

      IImageScannerFormatConfiguration *wrapper = IImageScannerFormatConfiguration::Unwrap<IImageScannerFormatConfiguration>(info.This());

      try 
      {
        
        ::Windows::Devices::Scanners::ImageScannerFormat winRtValue = static_cast<::Windows::Devices::Scanners::ImageScannerFormat>(value->Int32Value());

        wrapper->_instance->Format = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    


  private:
    ::Windows::Devices::Scanners::IImageScannerFormatConfiguration^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapIImageScannerFormatConfiguration(::Windows::Devices::Scanners::IImageScannerFormatConfiguration^ wintRtInstance);
    friend ::Windows::Devices::Scanners::IImageScannerFormatConfiguration^ UnwrapIImageScannerFormatConfiguration(Handle<Value> value);
    friend bool IsIImageScannerFormatConfigurationWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> IImageScannerFormatConfiguration::s_constructorTemplate;

  v8::Handle<v8::Value> WrapIImageScannerFormatConfiguration(::Windows::Devices::Scanners::IImageScannerFormatConfiguration^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(IImageScannerFormatConfiguration::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Devices::Scanners::IImageScannerFormatConfiguration^ UnwrapIImageScannerFormatConfiguration(Handle<Value> value)
  {
     return IImageScannerFormatConfiguration::Unwrap<IImageScannerFormatConfiguration>(value.As<Object>())->_instance;
  }

  void InitIImageScannerFormatConfiguration(Handle<Object> exports)
  {
    IImageScannerFormatConfiguration::Init(exports);
  }

  class ImageScannerAutoConfiguration : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ImageScannerAutoConfiguration"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("isFormatSupported"), FunctionTemplate::New(IsFormatSupported)->GetFunction());
      
                        
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("format"), FormatGetter, FormatSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("defaultFormat"), DefaultFormatGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ImageScannerAutoConfiguration"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ImageScannerAutoConfiguration(::Windows::Devices::Scanners::ImageScannerAutoConfiguration^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Devices::Scanners::ImageScannerAutoConfiguration^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerAutoConfiguration^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Devices::Scanners::ImageScannerAutoConfiguration^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ImageScannerAutoConfiguration *wrapperInstance = new ImageScannerAutoConfiguration(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> IsFormatSupported(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerAutoConfiguration^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerAutoConfiguration *wrapper = ImageScannerAutoConfiguration::Unwrap<ImageScannerAutoConfiguration>(args.This());

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Devices::Scanners::ImageScannerFormat arg0 = static_cast<::Windows::Devices::Scanners::ImageScannerFormat>(args[0]->Int32Value());
          
          bool result;
          result = wrapper->_instance->IsFormatSupported(arg0);
          return scope.Close(Boolean::New(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



    static Handle<Value> FormatGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerAutoConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerAutoConfiguration *wrapper = ImageScannerAutoConfiguration::Unwrap<ImageScannerAutoConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerFormat result = wrapper->_instance->Format;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void FormatSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerAutoConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerAutoConfiguration *wrapper = ImageScannerAutoConfiguration::Unwrap<ImageScannerAutoConfiguration>(info.This());

      try 
      {
        
        ::Windows::Devices::Scanners::ImageScannerFormat winRtValue = static_cast<::Windows::Devices::Scanners::ImageScannerFormat>(value->Int32Value());

        wrapper->_instance->Format = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> DefaultFormatGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerAutoConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerAutoConfiguration *wrapper = ImageScannerAutoConfiguration::Unwrap<ImageScannerAutoConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerFormat result = wrapper->_instance->DefaultFormat;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Devices::Scanners::ImageScannerAutoConfiguration^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapImageScannerAutoConfiguration(::Windows::Devices::Scanners::ImageScannerAutoConfiguration^ wintRtInstance);
    friend ::Windows::Devices::Scanners::ImageScannerAutoConfiguration^ UnwrapImageScannerAutoConfiguration(Handle<Value> value);
    friend bool IsImageScannerAutoConfigurationWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ImageScannerAutoConfiguration::s_constructorTemplate;

  v8::Handle<v8::Value> WrapImageScannerAutoConfiguration(::Windows::Devices::Scanners::ImageScannerAutoConfiguration^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ImageScannerAutoConfiguration::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Devices::Scanners::ImageScannerAutoConfiguration^ UnwrapImageScannerAutoConfiguration(Handle<Value> value)
  {
     return ImageScannerAutoConfiguration::Unwrap<ImageScannerAutoConfiguration>(value.As<Object>())->_instance;
  }

  void InitImageScannerAutoConfiguration(Handle<Object> exports)
  {
    ImageScannerAutoConfiguration::Init(exports);
  }

  class IImageScannerSourceConfiguration : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("IImageScannerSourceConfiguration"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("isAutoCroppingModeSupported"), FunctionTemplate::New(IsAutoCroppingModeSupported)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("isColorModeSupported"), FunctionTemplate::New(IsColorModeSupported)->GetFunction());
      
                        
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("actualResolution"), ActualResolutionGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("autoCroppingMode"), AutoCroppingModeGetter, AutoCroppingModeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("brightness"), BrightnessGetter, BrightnessSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("brightnessStep"), BrightnessStepGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("colorMode"), ColorModeGetter, ColorModeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("contrast"), ContrastGetter, ContrastSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("contrastStep"), ContrastStepGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("defaultBrightness"), DefaultBrightnessGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("defaultColorMode"), DefaultColorModeGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("defaultContrast"), DefaultContrastGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("desiredResolution"), DesiredResolutionGetter, DesiredResolutionSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("maxBrightness"), MaxBrightnessGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("maxContrast"), MaxContrastGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("maxResolution"), MaxResolutionGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("maxScanArea"), MaxScanAreaGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("minBrightness"), MinBrightnessGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("minContrast"), MinContrastGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("minResolution"), MinResolutionGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("minScanArea"), MinScanAreaGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("opticalResolution"), OpticalResolutionGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("selectedScanRegion"), SelectedScanRegionGetter, SelectedScanRegionSetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("IImageScannerSourceConfiguration"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    IImageScannerSourceConfiguration(::Windows::Devices::Scanners::IImageScannerSourceConfiguration^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Devices::Scanners::IImageScannerSourceConfiguration^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Devices::Scanners::IImageScannerSourceConfiguration^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      IImageScannerSourceConfiguration *wrapperInstance = new IImageScannerSourceConfiguration(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> IsAutoCroppingModeSupported(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(args.This());

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Devices::Scanners::ImageScannerAutoCroppingMode arg0 = static_cast<::Windows::Devices::Scanners::ImageScannerAutoCroppingMode>(args[0]->Int32Value());
          
          bool result;
          result = wrapper->_instance->IsAutoCroppingModeSupported(arg0);
          return scope.Close(Boolean::New(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> IsColorModeSupported(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(args.This());

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Devices::Scanners::ImageScannerColorMode arg0 = static_cast<::Windows::Devices::Scanners::ImageScannerColorMode>(args[0]->Int32Value());
          
          bool result;
          result = wrapper->_instance->IsColorModeSupported(arg0);
          return scope.Close(Boolean::New(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



    static Handle<Value> ActualResolutionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerResolution result = wrapper->_instance->ActualResolution;
        return scope.Close(ImageScannerResolutionToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> AutoCroppingModeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerAutoCroppingMode result = wrapper->_instance->AutoCroppingMode;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void AutoCroppingModeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return;
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        
        ::Windows::Devices::Scanners::ImageScannerAutoCroppingMode winRtValue = static_cast<::Windows::Devices::Scanners::ImageScannerAutoCroppingMode>(value->Int32Value());

        wrapper->_instance->AutoCroppingMode = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> BrightnessGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->Brightness;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void BrightnessSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return;
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        
        int winRtValue = static_cast<int>(value->Int32Value());

        wrapper->_instance->Brightness = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> BrightnessStepGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->BrightnessStep;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ColorModeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerColorMode result = wrapper->_instance->ColorMode;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ColorModeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return;
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        
        ::Windows::Devices::Scanners::ImageScannerColorMode winRtValue = static_cast<::Windows::Devices::Scanners::ImageScannerColorMode>(value->Int32Value());

        wrapper->_instance->ColorMode = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ContrastGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->Contrast;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ContrastSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return;
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        
        int winRtValue = static_cast<int>(value->Int32Value());

        wrapper->_instance->Contrast = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ContrastStepGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->ContrastStep;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> DefaultBrightnessGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->DefaultBrightness;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> DefaultColorModeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerColorMode result = wrapper->_instance->DefaultColorMode;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> DefaultContrastGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->DefaultContrast;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> DesiredResolutionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerResolution result = wrapper->_instance->DesiredResolution;
        return scope.Close(ImageScannerResolutionToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void DesiredResolutionSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!IsImageScannerResolutionJsObject(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return;
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        
        ::Windows::Devices::Scanners::ImageScannerResolution winRtValue = ImageScannerResolutionFromJsObject(value);

        wrapper->_instance->DesiredResolution = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> MaxBrightnessGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->MaxBrightness;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MaxContrastGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->MaxContrast;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MaxResolutionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerResolution result = wrapper->_instance->MaxResolution;
        return scope.Close(ImageScannerResolutionToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MaxScanAreaGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        ::Windows::Foundation::Size result = wrapper->_instance->MaxScanArea;
        return scope.Close(NodeRT::Utils::SizeToJs(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MinBrightnessGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->MinBrightness;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MinContrastGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->MinContrast;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MinResolutionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerResolution result = wrapper->_instance->MinResolution;
        return scope.Close(ImageScannerResolutionToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MinScanAreaGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        ::Windows::Foundation::Size result = wrapper->_instance->MinScanArea;
        return scope.Close(NodeRT::Utils::SizeToJs(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> OpticalResolutionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerResolution result = wrapper->_instance->OpticalResolution;
        return scope.Close(ImageScannerResolutionToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> SelectedScanRegionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        ::Windows::Foundation::Rect result = wrapper->_instance->SelectedScanRegion;
        return scope.Close(NodeRT::Utils::RectToJs(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void SelectedScanRegionSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsRect(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::IImageScannerSourceConfiguration^>(info.This()))
      {
        return;
      }

      IImageScannerSourceConfiguration *wrapper = IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(info.This());

      try 
      {
        
        ::Windows::Foundation::Rect winRtValue = NodeRT::Utils::RectFromJs(value);

        wrapper->_instance->SelectedScanRegion = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    


  private:
    ::Windows::Devices::Scanners::IImageScannerSourceConfiguration^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapIImageScannerSourceConfiguration(::Windows::Devices::Scanners::IImageScannerSourceConfiguration^ wintRtInstance);
    friend ::Windows::Devices::Scanners::IImageScannerSourceConfiguration^ UnwrapIImageScannerSourceConfiguration(Handle<Value> value);
    friend bool IsIImageScannerSourceConfigurationWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> IImageScannerSourceConfiguration::s_constructorTemplate;

  v8::Handle<v8::Value> WrapIImageScannerSourceConfiguration(::Windows::Devices::Scanners::IImageScannerSourceConfiguration^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(IImageScannerSourceConfiguration::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Devices::Scanners::IImageScannerSourceConfiguration^ UnwrapIImageScannerSourceConfiguration(Handle<Value> value)
  {
     return IImageScannerSourceConfiguration::Unwrap<IImageScannerSourceConfiguration>(value.As<Object>())->_instance;
  }

  void InitIImageScannerSourceConfiguration(Handle<Object> exports)
  {
    IImageScannerSourceConfiguration::Init(exports);
  }

  class ImageScannerFlatbedConfiguration : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ImageScannerFlatbedConfiguration"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("isFormatSupported"), FunctionTemplate::New(IsFormatSupported)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("isAutoCroppingModeSupported"), FunctionTemplate::New(IsAutoCroppingModeSupported)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("isColorModeSupported"), FunctionTemplate::New(IsColorModeSupported)->GetFunction());
      
                        
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("format"), FormatGetter, FormatSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("defaultFormat"), DefaultFormatGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("selectedScanRegion"), SelectedScanRegionGetter, SelectedScanRegionSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("desiredResolution"), DesiredResolutionGetter, DesiredResolutionSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("contrast"), ContrastGetter, ContrastSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("colorMode"), ColorModeGetter, ColorModeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("brightness"), BrightnessGetter, BrightnessSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("autoCroppingMode"), AutoCroppingModeGetter, AutoCroppingModeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("actualResolution"), ActualResolutionGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("brightnessStep"), BrightnessStepGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("contrastStep"), ContrastStepGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("defaultBrightness"), DefaultBrightnessGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("defaultColorMode"), DefaultColorModeGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("defaultContrast"), DefaultContrastGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("maxBrightness"), MaxBrightnessGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("maxContrast"), MaxContrastGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("maxResolution"), MaxResolutionGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("maxScanArea"), MaxScanAreaGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("minBrightness"), MinBrightnessGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("minContrast"), MinContrastGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("minResolution"), MinResolutionGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("minScanArea"), MinScanAreaGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("opticalResolution"), OpticalResolutionGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ImageScannerFlatbedConfiguration"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ImageScannerFlatbedConfiguration(::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ImageScannerFlatbedConfiguration *wrapperInstance = new ImageScannerFlatbedConfiguration(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> IsFormatSupported(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(args.This());

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Devices::Scanners::ImageScannerFormat arg0 = static_cast<::Windows::Devices::Scanners::ImageScannerFormat>(args[0]->Int32Value());
          
          bool result;
          result = wrapper->_instance->IsFormatSupported(arg0);
          return scope.Close(Boolean::New(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> IsAutoCroppingModeSupported(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(args.This());

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Devices::Scanners::ImageScannerAutoCroppingMode arg0 = static_cast<::Windows::Devices::Scanners::ImageScannerAutoCroppingMode>(args[0]->Int32Value());
          
          bool result;
          result = wrapper->_instance->IsAutoCroppingModeSupported(arg0);
          return scope.Close(Boolean::New(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> IsColorModeSupported(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(args.This());

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Devices::Scanners::ImageScannerColorMode arg0 = static_cast<::Windows::Devices::Scanners::ImageScannerColorMode>(args[0]->Int32Value());
          
          bool result;
          result = wrapper->_instance->IsColorModeSupported(arg0);
          return scope.Close(Boolean::New(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



    static Handle<Value> FormatGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerFormat result = wrapper->_instance->Format;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void FormatSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        
        ::Windows::Devices::Scanners::ImageScannerFormat winRtValue = static_cast<::Windows::Devices::Scanners::ImageScannerFormat>(value->Int32Value());

        wrapper->_instance->Format = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> DefaultFormatGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerFormat result = wrapper->_instance->DefaultFormat;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> SelectedScanRegionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        ::Windows::Foundation::Rect result = wrapper->_instance->SelectedScanRegion;
        return scope.Close(NodeRT::Utils::RectToJs(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void SelectedScanRegionSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsRect(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        
        ::Windows::Foundation::Rect winRtValue = NodeRT::Utils::RectFromJs(value);

        wrapper->_instance->SelectedScanRegion = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> DesiredResolutionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerResolution result = wrapper->_instance->DesiredResolution;
        return scope.Close(ImageScannerResolutionToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void DesiredResolutionSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!IsImageScannerResolutionJsObject(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        
        ::Windows::Devices::Scanners::ImageScannerResolution winRtValue = ImageScannerResolutionFromJsObject(value);

        wrapper->_instance->DesiredResolution = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ContrastGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->Contrast;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ContrastSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        
        int winRtValue = static_cast<int>(value->Int32Value());

        wrapper->_instance->Contrast = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ColorModeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerColorMode result = wrapper->_instance->ColorMode;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ColorModeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        
        ::Windows::Devices::Scanners::ImageScannerColorMode winRtValue = static_cast<::Windows::Devices::Scanners::ImageScannerColorMode>(value->Int32Value());

        wrapper->_instance->ColorMode = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> BrightnessGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->Brightness;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void BrightnessSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        
        int winRtValue = static_cast<int>(value->Int32Value());

        wrapper->_instance->Brightness = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> AutoCroppingModeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerAutoCroppingMode result = wrapper->_instance->AutoCroppingMode;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void AutoCroppingModeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        
        ::Windows::Devices::Scanners::ImageScannerAutoCroppingMode winRtValue = static_cast<::Windows::Devices::Scanners::ImageScannerAutoCroppingMode>(value->Int32Value());

        wrapper->_instance->AutoCroppingMode = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ActualResolutionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerResolution result = wrapper->_instance->ActualResolution;
        return scope.Close(ImageScannerResolutionToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> BrightnessStepGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->BrightnessStep;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ContrastStepGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->ContrastStep;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> DefaultBrightnessGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->DefaultBrightness;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> DefaultColorModeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerColorMode result = wrapper->_instance->DefaultColorMode;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> DefaultContrastGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->DefaultContrast;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MaxBrightnessGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->MaxBrightness;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MaxContrastGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->MaxContrast;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MaxResolutionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerResolution result = wrapper->_instance->MaxResolution;
        return scope.Close(ImageScannerResolutionToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MaxScanAreaGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        ::Windows::Foundation::Size result = wrapper->_instance->MaxScanArea;
        return scope.Close(NodeRT::Utils::SizeToJs(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MinBrightnessGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->MinBrightness;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MinContrastGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->MinContrast;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MinResolutionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerResolution result = wrapper->_instance->MinResolution;
        return scope.Close(ImageScannerResolutionToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MinScanAreaGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        ::Windows::Foundation::Size result = wrapper->_instance->MinScanArea;
        return scope.Close(NodeRT::Utils::SizeToJs(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> OpticalResolutionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFlatbedConfiguration *wrapper = ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerResolution result = wrapper->_instance->OpticalResolution;
        return scope.Close(ImageScannerResolutionToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapImageScannerFlatbedConfiguration(::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^ wintRtInstance);
    friend ::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^ UnwrapImageScannerFlatbedConfiguration(Handle<Value> value);
    friend bool IsImageScannerFlatbedConfigurationWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ImageScannerFlatbedConfiguration::s_constructorTemplate;

  v8::Handle<v8::Value> WrapImageScannerFlatbedConfiguration(::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ImageScannerFlatbedConfiguration::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^ UnwrapImageScannerFlatbedConfiguration(Handle<Value> value)
  {
     return ImageScannerFlatbedConfiguration::Unwrap<ImageScannerFlatbedConfiguration>(value.As<Object>())->_instance;
  }

  void InitImageScannerFlatbedConfiguration(Handle<Object> exports)
  {
    ImageScannerFlatbedConfiguration::Init(exports);
  }

  class ImageScannerFeederConfiguration : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ImageScannerFeederConfiguration"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("isFormatSupported"), FunctionTemplate::New(IsFormatSupported)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("isAutoCroppingModeSupported"), FunctionTemplate::New(IsAutoCroppingModeSupported)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("isColorModeSupported"), FunctionTemplate::New(IsColorModeSupported)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("isPageSizeSupported"), FunctionTemplate::New(IsPageSizeSupported)->GetFunction());
      
                        
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("format"), FormatGetter, FormatSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("defaultFormat"), DefaultFormatGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("selectedScanRegion"), SelectedScanRegionGetter, SelectedScanRegionSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("colorMode"), ColorModeGetter, ColorModeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("desiredResolution"), DesiredResolutionGetter, DesiredResolutionSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("brightness"), BrightnessGetter, BrightnessSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("autoCroppingMode"), AutoCroppingModeGetter, AutoCroppingModeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("contrast"), ContrastGetter, ContrastSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("brightnessStep"), BrightnessStepGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("contrastStep"), ContrastStepGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("defaultBrightness"), DefaultBrightnessGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("defaultColorMode"), DefaultColorModeGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("defaultContrast"), DefaultContrastGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("maxBrightness"), MaxBrightnessGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("maxContrast"), MaxContrastGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("maxResolution"), MaxResolutionGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("maxScanArea"), MaxScanAreaGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("minBrightness"), MinBrightnessGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("minContrast"), MinContrastGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("minResolution"), MinResolutionGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("minScanArea"), MinScanAreaGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("opticalResolution"), OpticalResolutionGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("actualResolution"), ActualResolutionGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("scanAhead"), ScanAheadGetter, ScanAheadSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("pageSize"), PageSizeGetter, PageSizeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("pageOrientation"), PageOrientationGetter, PageOrientationSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("maxNumberOfPages"), MaxNumberOfPagesGetter, MaxNumberOfPagesSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("duplex"), DuplexGetter, DuplexSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("autoDetectPageSize"), AutoDetectPageSizeGetter, AutoDetectPageSizeSetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("canAutoDetectPageSize"), CanAutoDetectPageSizeGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("canScanAhead"), CanScanAheadGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("canScanDuplex"), CanScanDuplexGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("pageSizeDimensions"), PageSizeDimensionsGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ImageScannerFeederConfiguration"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ImageScannerFeederConfiguration(::Windows::Devices::Scanners::ImageScannerFeederConfiguration^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Devices::Scanners::ImageScannerFeederConfiguration^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Devices::Scanners::ImageScannerFeederConfiguration^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ImageScannerFeederConfiguration *wrapperInstance = new ImageScannerFeederConfiguration(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  
    static Handle<Value> IsFormatSupported(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(args.This());

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Devices::Scanners::ImageScannerFormat arg0 = static_cast<::Windows::Devices::Scanners::ImageScannerFormat>(args[0]->Int32Value());
          
          bool result;
          result = wrapper->_instance->IsFormatSupported(arg0);
          return scope.Close(Boolean::New(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> IsAutoCroppingModeSupported(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(args.This());

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Devices::Scanners::ImageScannerAutoCroppingMode arg0 = static_cast<::Windows::Devices::Scanners::ImageScannerAutoCroppingMode>(args[0]->Int32Value());
          
          bool result;
          result = wrapper->_instance->IsAutoCroppingModeSupported(arg0);
          return scope.Close(Boolean::New(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> IsColorModeSupported(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(args.This());

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Devices::Scanners::ImageScannerColorMode arg0 = static_cast<::Windows::Devices::Scanners::ImageScannerColorMode>(args[0]->Int32Value());
          
          bool result;
          result = wrapper->_instance->IsColorModeSupported(arg0);
          return scope.Close(Boolean::New(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> IsPageSizeSupported(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(args.This());

      if (args.Length() == 2
        && args[0]->IsInt32()
        && args[1]->IsInt32())
      {
        try
        {
          ::Windows::Graphics::Printing::PrintMediaSize arg0 = static_cast<::Windows::Graphics::Printing::PrintMediaSize>(args[0]->Int32Value());
          ::Windows::Graphics::Printing::PrintOrientation arg1 = static_cast<::Windows::Graphics::Printing::PrintOrientation>(args[1]->Int32Value());
          
          bool result;
          result = wrapper->_instance->IsPageSizeSupported(arg0, arg1);
          return scope.Close(Boolean::New(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }



    static Handle<Value> FormatGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerFormat result = wrapper->_instance->Format;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void FormatSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        
        ::Windows::Devices::Scanners::ImageScannerFormat winRtValue = static_cast<::Windows::Devices::Scanners::ImageScannerFormat>(value->Int32Value());

        wrapper->_instance->Format = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> DefaultFormatGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerFormat result = wrapper->_instance->DefaultFormat;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> SelectedScanRegionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        ::Windows::Foundation::Rect result = wrapper->_instance->SelectedScanRegion;
        return scope.Close(NodeRT::Utils::RectToJs(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void SelectedScanRegionSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsRect(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        
        ::Windows::Foundation::Rect winRtValue = NodeRT::Utils::RectFromJs(value);

        wrapper->_instance->SelectedScanRegion = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ColorModeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerColorMode result = wrapper->_instance->ColorMode;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ColorModeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        
        ::Windows::Devices::Scanners::ImageScannerColorMode winRtValue = static_cast<::Windows::Devices::Scanners::ImageScannerColorMode>(value->Int32Value());

        wrapper->_instance->ColorMode = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> DesiredResolutionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerResolution result = wrapper->_instance->DesiredResolution;
        return scope.Close(ImageScannerResolutionToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void DesiredResolutionSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!IsImageScannerResolutionJsObject(value))
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        
        ::Windows::Devices::Scanners::ImageScannerResolution winRtValue = ImageScannerResolutionFromJsObject(value);

        wrapper->_instance->DesiredResolution = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> BrightnessGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->Brightness;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void BrightnessSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        
        int winRtValue = static_cast<int>(value->Int32Value());

        wrapper->_instance->Brightness = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> AutoCroppingModeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerAutoCroppingMode result = wrapper->_instance->AutoCroppingMode;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void AutoCroppingModeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        
        ::Windows::Devices::Scanners::ImageScannerAutoCroppingMode winRtValue = static_cast<::Windows::Devices::Scanners::ImageScannerAutoCroppingMode>(value->Int32Value());

        wrapper->_instance->AutoCroppingMode = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> ContrastGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->Contrast;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ContrastSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        
        int winRtValue = static_cast<int>(value->Int32Value());

        wrapper->_instance->Contrast = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> BrightnessStepGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->BrightnessStep;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ContrastStepGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->ContrastStep;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> DefaultBrightnessGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->DefaultBrightness;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> DefaultColorModeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerColorMode result = wrapper->_instance->DefaultColorMode;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> DefaultContrastGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->DefaultContrast;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MaxBrightnessGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->MaxBrightness;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MaxContrastGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->MaxContrast;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MaxResolutionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerResolution result = wrapper->_instance->MaxResolution;
        return scope.Close(ImageScannerResolutionToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MaxScanAreaGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        ::Windows::Foundation::Size result = wrapper->_instance->MaxScanArea;
        return scope.Close(NodeRT::Utils::SizeToJs(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MinBrightnessGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->MinBrightness;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MinContrastGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        int result = wrapper->_instance->MinContrast;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MinResolutionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerResolution result = wrapper->_instance->MinResolution;
        return scope.Close(ImageScannerResolutionToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> MinScanAreaGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        ::Windows::Foundation::Size result = wrapper->_instance->MinScanArea;
        return scope.Close(NodeRT::Utils::SizeToJs(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> OpticalResolutionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerResolution result = wrapper->_instance->OpticalResolution;
        return scope.Close(ImageScannerResolutionToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ActualResolutionGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerResolution result = wrapper->_instance->ActualResolution;
        return scope.Close(ImageScannerResolutionToJsObject(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> ScanAheadGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        bool result = wrapper->_instance->ScanAhead;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void ScanAheadSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->ScanAhead = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> PageSizeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        ::Windows::Graphics::Printing::PrintMediaSize result = wrapper->_instance->PageSize;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void PageSizeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        
        ::Windows::Graphics::Printing::PrintMediaSize winRtValue = static_cast<::Windows::Graphics::Printing::PrintMediaSize>(value->Int32Value());

        wrapper->_instance->PageSize = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> PageOrientationGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        ::Windows::Graphics::Printing::PrintOrientation result = wrapper->_instance->PageOrientation;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void PageOrientationSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsInt32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        
        ::Windows::Graphics::Printing::PrintOrientation winRtValue = static_cast<::Windows::Graphics::Printing::PrintOrientation>(value->Int32Value());

        wrapper->_instance->PageOrientation = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> MaxNumberOfPagesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        unsigned int result = wrapper->_instance->MaxNumberOfPages;
        return scope.Close(Integer::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void MaxNumberOfPagesSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsUint32())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        
        unsigned int winRtValue = static_cast<unsigned int>(value->IntegerValue());

        wrapper->_instance->MaxNumberOfPages = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> DuplexGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        bool result = wrapper->_instance->Duplex;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void DuplexSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->Duplex = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> AutoDetectPageSizeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        bool result = wrapper->_instance->AutoDetectPageSize;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static void AutoDetectPageSizeSetter(Local<String> property, Local<Value> value, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!value->IsBoolean())
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Value to set is of unexpected type")));
        return;
      }

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return;
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        
        bool winRtValue = value->BooleanValue();

        wrapper->_instance->AutoDetectPageSize = winRtValue;
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
      }
    }
    
    static Handle<Value> CanAutoDetectPageSizeGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        bool result = wrapper->_instance->CanAutoDetectPageSize;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> CanScanAheadGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        bool result = wrapper->_instance->CanScanAhead;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> CanScanDuplexGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        bool result = wrapper->_instance->CanScanDuplex;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> PageSizeDimensionsGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerFeederConfiguration^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerFeederConfiguration *wrapper = ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(info.This());

      try 
      {
        ::Windows::Foundation::Size result = wrapper->_instance->PageSizeDimensions;
        return scope.Close(NodeRT::Utils::SizeToJs(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Devices::Scanners::ImageScannerFeederConfiguration^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapImageScannerFeederConfiguration(::Windows::Devices::Scanners::ImageScannerFeederConfiguration^ wintRtInstance);
    friend ::Windows::Devices::Scanners::ImageScannerFeederConfiguration^ UnwrapImageScannerFeederConfiguration(Handle<Value> value);
    friend bool IsImageScannerFeederConfigurationWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ImageScannerFeederConfiguration::s_constructorTemplate;

  v8::Handle<v8::Value> WrapImageScannerFeederConfiguration(::Windows::Devices::Scanners::ImageScannerFeederConfiguration^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ImageScannerFeederConfiguration::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Devices::Scanners::ImageScannerFeederConfiguration^ UnwrapImageScannerFeederConfiguration(Handle<Value> value)
  {
     return ImageScannerFeederConfiguration::Unwrap<ImageScannerFeederConfiguration>(value.As<Object>())->_instance;
  }

  void InitImageScannerFeederConfiguration(Handle<Object> exports)
  {
    ImageScannerFeederConfiguration::Init(exports);
  }

  class ImageScannerScanResult : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ImageScannerScanResult"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("scannedFiles"), ScannedFilesGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ImageScannerScanResult"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ImageScannerScanResult(::Windows::Devices::Scanners::ImageScannerScanResult^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Devices::Scanners::ImageScannerScanResult^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerScanResult^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Devices::Scanners::ImageScannerScanResult^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ImageScannerScanResult *wrapperInstance = new ImageScannerScanResult(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> ScannedFilesGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerScanResult^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerScanResult *wrapper = ImageScannerScanResult::Unwrap<ImageScannerScanResult>(info.This());

      try 
      {
        ::Windows::Foundation::Collections::IVectorView<::Windows::Storage::StorageFile^>^ result = wrapper->_instance->ScannedFiles;
        return scope.Close(NodeRT::Collections::VectorViewWrapper<::Windows::Storage::StorageFile^>::CreateVectorViewWrapper(result, 
            [](::Windows::Storage::StorageFile^ val) -> Handle<Value> {
              return NodeRT::Utils::CreateExternalWinRTObject("Windows.Storage", "StorageFile", val);
            },
            [](Handle<Value> value) -> bool {
              return NodeRT::Utils::IsWinRtWrapperOf<::Windows::Storage::StorageFile^>(value);
            },
            [](Handle<Value> value) -> ::Windows::Storage::StorageFile^ {
              return dynamic_cast<::Windows::Storage::StorageFile^>(NodeRT::Utils::GetObjectInstance(value));
            }
          ));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Devices::Scanners::ImageScannerScanResult^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapImageScannerScanResult(::Windows::Devices::Scanners::ImageScannerScanResult^ wintRtInstance);
    friend ::Windows::Devices::Scanners::ImageScannerScanResult^ UnwrapImageScannerScanResult(Handle<Value> value);
    friend bool IsImageScannerScanResultWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ImageScannerScanResult::s_constructorTemplate;

  v8::Handle<v8::Value> WrapImageScannerScanResult(::Windows::Devices::Scanners::ImageScannerScanResult^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ImageScannerScanResult::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Devices::Scanners::ImageScannerScanResult^ UnwrapImageScannerScanResult(Handle<Value> value)
  {
     return ImageScannerScanResult::Unwrap<ImageScannerScanResult>(value.As<Object>())->_instance;
  }

  void InitImageScannerScanResult(Handle<Object> exports)
  {
    ImageScannerScanResult::Init(exports);
  }

  class ImageScannerPreviewResult : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ImageScannerPreviewResult"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
                              
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("format"), FormatGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("succeeded"), SucceededGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();


      exports->Set(String::NewSymbol("ImageScannerPreviewResult"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ImageScannerPreviewResult(::Windows::Devices::Scanners::ImageScannerPreviewResult^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Devices::Scanners::ImageScannerPreviewResult^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerPreviewResult^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Devices::Scanners::ImageScannerPreviewResult^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ImageScannerPreviewResult *wrapperInstance = new ImageScannerPreviewResult(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


  



    static Handle<Value> FormatGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerPreviewResult^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerPreviewResult *wrapper = ImageScannerPreviewResult::Unwrap<ImageScannerPreviewResult>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerFormat result = wrapper->_instance->Format;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> SucceededGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScannerPreviewResult^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScannerPreviewResult *wrapper = ImageScannerPreviewResult::Unwrap<ImageScannerPreviewResult>(info.This());

      try 
      {
        bool result = wrapper->_instance->Succeeded;
        return scope.Close(Boolean::New(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Devices::Scanners::ImageScannerPreviewResult^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapImageScannerPreviewResult(::Windows::Devices::Scanners::ImageScannerPreviewResult^ wintRtInstance);
    friend ::Windows::Devices::Scanners::ImageScannerPreviewResult^ UnwrapImageScannerPreviewResult(Handle<Value> value);
    friend bool IsImageScannerPreviewResultWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ImageScannerPreviewResult::s_constructorTemplate;

  v8::Handle<v8::Value> WrapImageScannerPreviewResult(::Windows::Devices::Scanners::ImageScannerPreviewResult^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ImageScannerPreviewResult::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Devices::Scanners::ImageScannerPreviewResult^ UnwrapImageScannerPreviewResult(Handle<Value> value)
  {
     return ImageScannerPreviewResult::Unwrap<ImageScannerPreviewResult>(value.As<Object>())->_instance;
  }

  void InitImageScannerPreviewResult(Handle<Object> exports)
  {
    ImageScannerPreviewResult::Init(exports);
  }

  class ImageScanner : public WrapperBase
  {
  public:    
    static v8::Handle<v8::Value> Init(const Handle<Object> exports)
    {
      HandleScope scope;
      
      s_constructorTemplate = Persistent<FunctionTemplate>::New(FunctionTemplate::New(New));
      s_constructorTemplate->SetClassName(String::NewSymbol("ImageScanner"));
      s_constructorTemplate->InstanceTemplate()->SetInternalFieldCount(1);
      
      Handle<Value> asyncSymbol = String::NewSymbol("__winRtAsync__");
      Handle<Function> func;
            
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("isScanSourceSupported"), FunctionTemplate::New(IsScanSourceSupported)->GetFunction());
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("isPreviewSupported"), FunctionTemplate::New(IsPreviewSupported)->GetFunction());
      
            
      func = FunctionTemplate::New(ScanPreviewToStreamAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("scanPreviewToStreamAsync"), func);
      func = FunctionTemplate::New(ScanFilesToFolderAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      s_constructorTemplate->PrototypeTemplate()->Set(String::NewSymbol("scanFilesToFolderAsync"), func);
      
                  
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("autoConfiguration"), AutoConfigurationGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("defaultScanSource"), DefaultScanSourceGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("deviceId"), DeviceIdGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("feederConfiguration"), FeederConfigurationGetter);
      s_constructorTemplate->PrototypeTemplate()->SetAccessor(String::NewSymbol("flatbedConfiguration"), FlatbedConfigurationGetter);
      
      Local<Function> constructor = s_constructorTemplate->GetFunction();

      constructor->Set(String::NewSymbol("getDeviceSelector"), FunctionTemplate::New(GetDeviceSelector)->GetFunction());
      func = FunctionTemplate::New(FromIdAsync)->GetFunction();
      func->Set(asyncSymbol, True(), PropertyAttribute::DontEnum);
      constructor->Set(String::NewSymbol("fromIdAsync"), func);

      exports->Set(String::NewSymbol("ImageScanner"), constructor);
      return scope.Close(Undefined());
    }


    virtual ::Platform::Object^ GetObjectInstance() const override
    {
      return _instance;
    }

  private:
    
    ImageScanner(::Windows::Devices::Scanners::ImageScanner^ instance)
    {
      _instance = instance;
    }
    
    
    static v8::Handle<v8::Value> New(const v8::Arguments& args)
    {
      HandleScope scope;

      // in case the constructor was called without the new operator
      if (!s_constructorTemplate->HasInstance(args.This()))
      {
        if (args.Length() > 0)
        {
          std::unique_ptr<Handle<Value> []> constructorArgs(new Handle<Value>[args.Length()]);

          Handle<Value> *argsPtr = constructorArgs.get();
          for (int i = 0; i < args.Length(); i++)
          {
            argsPtr[i] = args[i];
          }

          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), constructorArgs.get());
        }
        else
        {
          return s_constructorTemplate->GetFunction()->CallAsConstructor(args.Length(), nullptr);
        }
      }
      
      ::Windows::Devices::Scanners::ImageScanner^ winRtInstance;


      if (args.Length() == 1 && OpaqueWrapper::IsOpaqueWrapper(args[0]) &&
        NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScanner^>(args[0]))
      {
        try 
        {
          winRtInstance = (::Windows::Devices::Scanners::ImageScanner^) NodeRT::Utils::GetObjectInstance(args[0]);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Invalid arguments, no suitable constructor found")));
        return scope.Close(Undefined());
      }

      args.This()->SetHiddenValue(String::NewSymbol("__winRtInstance__"), True());

      ImageScanner *wrapperInstance = new ImageScanner(winRtInstance);
      wrapperInstance->Wrap(args.This());

      return args.This();
    }


    static Handle<Value> ScanPreviewToStreamAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScanner^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      ImageScanner *wrapper = ImageScanner::Unwrap<ImageScanner>(args.This());

      ::Windows::Foundation::IAsyncOperation<::Windows::Devices::Scanners::ImageScannerPreviewResult^>^ op;
    

      if (args.Length() == 3
        && args[0]->IsInt32()
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Storage::Streams::IRandomAccessStream^>(args[1]))
      {
        try
        {
          ::Windows::Devices::Scanners::ImageScannerScanSource arg0 = static_cast<::Windows::Devices::Scanners::ImageScannerScanSource>(args[0]->Int32Value());
          ::Windows::Storage::Streams::IRandomAccessStream^ arg1 = dynamic_cast<::Windows::Storage::Streams::IRandomAccessStream^>(NodeRT::Utils::GetObjectInstance(args[1]));
          
          op = wrapper->_instance->ScanPreviewToStreamAsync(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<::Windows::Devices::Scanners::ImageScannerPreviewResult^> t) 
      {	
        try
        {
          auto result = t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [result](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            TryCatch tryCatch;
            Handle<Value> error; 
            Handle<Value> arg1 = WrapImageScannerPreviewResult(result);
            if (tryCatch.HasCaught())
            {
              error = tryCatch.Exception()->ToObject();
            }
            else 
            {
              error = Undefined();
            }
            if (arg1.IsEmpty()) arg1 = Undefined();
            Handle<Value> args[] = {error, arg1};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
    static Handle<Value> ScanFilesToFolderAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScanner^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      ImageScanner *wrapper = ImageScanner::Unwrap<ImageScanner>(args.This());

      ::Windows::Foundation::IAsyncOperationWithProgress<::Windows::Devices::Scanners::ImageScannerScanResult^, unsigned int>^ op;
    

      if (args.Length() == 3
        && args[0]->IsInt32()
        && NodeRT::Utils::IsWinRtWrapperOf<::Windows::Storage::StorageFolder^>(args[1]))
      {
        try
        {
          ::Windows::Devices::Scanners::ImageScannerScanSource arg0 = static_cast<::Windows::Devices::Scanners::ImageScannerScanSource>(args[0]->Int32Value());
          ::Windows::Storage::StorageFolder^ arg1 = dynamic_cast<::Windows::Storage::StorageFolder^>(NodeRT::Utils::GetObjectInstance(args[1]));
          
          op = wrapper->_instance->ScanFilesToFolderAsync(arg0,arg1);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<::Windows::Devices::Scanners::ImageScannerScanResult^> t) 
      {	
        try
        {
          auto result = t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [result](NodeUtils::InvokeCallbackDelegate invokeCallback) {


            TryCatch tryCatch;
            Handle<Value> error; 
            Handle<Value> arg1 = WrapImageScannerScanResult(result);
            if (tryCatch.HasCaught())
            {
              error = tryCatch.Exception()->ToObject();
            }
            else 
            {
              error = Undefined();
            }
            if (arg1.IsEmpty()) arg1 = Undefined();
            Handle<Value> args[] = {error, arg1};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
             
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }
  
    static Handle<Value> IsScanSourceSupported(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScanner^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScanner *wrapper = ImageScanner::Unwrap<ImageScanner>(args.This());

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Devices::Scanners::ImageScannerScanSource arg0 = static_cast<::Windows::Devices::Scanners::ImageScannerScanSource>(args[0]->Int32Value());
          
          bool result;
          result = wrapper->_instance->IsScanSourceSupported(arg0);
          return scope.Close(Boolean::New(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }
    static Handle<Value> IsPreviewSupported(const v8::Arguments& args)
    {
      HandleScope scope;

      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScanner^>(args.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScanner *wrapper = ImageScanner::Unwrap<ImageScanner>(args.This());

      if (args.Length() == 1
        && args[0]->IsInt32())
      {
        try
        {
          ::Windows::Devices::Scanners::ImageScannerScanSource arg0 = static_cast<::Windows::Devices::Scanners::ImageScannerScanSource>(args[0]->Int32Value());
          
          bool result;
          result = wrapper->_instance->IsPreviewSupported(arg0);
          return scope.Close(Boolean::New(result));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }

    static Handle<Value> FromIdAsync(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 0 || !args[args.Length() -1]->IsFunction())
      {
          ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: No callback was given")));
          return scope.Close(Undefined());
      }

      ::Windows::Foundation::IAsyncOperation<::Windows::Devices::Scanners::ImageScanner^>^ op;
      

      if (args.Length() == 2
        && args[0]->IsString())
      {
        try
        {
          Platform::String^ arg0 = ref new Platform::String(NodeRT::Utils::StringToWchar(v8::String::Value(args[0])));
          
          op = ::Windows::Devices::Scanners::ImageScanner::FromIdAsync(arg0);
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }
    
      auto opTask = create_task(op);
      uv_async_t* asyncToken = NodeUtils::Async::GetAsyncToken(args[args.Length() -1].As<Function>());

      opTask.then( [asyncToken] (task<::Windows::Devices::Scanners::ImageScanner^> t) 
      {	
        try
        {
          auto result = t.get();
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [result](NodeUtils::InvokeCallbackDelegate invokeCallback) {

            
            TryCatch tryCatch;
            Handle<Value> error; 
            Handle<Value> arg1 = WrapImageScanner(result);
            if (tryCatch.HasCaught())
            {
              error = tryCatch.Exception()->ToObject();
            }
            else 
            {
              error = Undefined();
            }
            if (arg1.IsEmpty()) arg1 = Undefined();
            Handle<Value> args[] = {error, arg1};

            invokeCallback(_countof(args), args);
          });
        }
        catch (Platform::Exception^ exception)
        {
          NodeUtils::Async::RunCallbackOnMain(asyncToken, [exception](NodeUtils::InvokeCallbackDelegate invokeCallback) {
          
            Handle<Value> error = NodeRT::Utils::WinRtExceptionToJsError(exception);
        
            Handle<Value> args[] = {error};
            invokeCallback(_countof(args), args);
          });
        }  		
      });

      return scope.Close(Undefined());
    }

    static Handle<Value> GetDeviceSelector(const v8::Arguments& args)
    {
      HandleScope scope;

      if (args.Length() == 0)
      {
        try
        {
          Platform::String^ result;
          result = ::Windows::Devices::Scanners::ImageScanner::GetDeviceSelector();
          return scope.Close(NodeRT::Utils::NewString(result->Data()));
        }
        catch (Platform::Exception ^exception)
        {
          NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
          return scope.Close(Undefined());
        }
      }
      else 
      {
        ThrowException(Exception::Error(NodeRT::Utils::NewString(L"Bad arguments: no suitable overload found")));
        return scope.Close(Undefined());
      }

      return scope.Close(Undefined());
    }

    static Handle<Value> AutoConfigurationGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScanner^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScanner *wrapper = ImageScanner::Unwrap<ImageScanner>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerAutoConfiguration^ result = wrapper->_instance->AutoConfiguration;
        return scope.Close(WrapImageScannerAutoConfiguration(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> DefaultScanSourceGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScanner^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScanner *wrapper = ImageScanner::Unwrap<ImageScanner>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerScanSource result = wrapper->_instance->DefaultScanSource;
        return scope.Close(Integer::New(static_cast<int>(result)));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> DeviceIdGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScanner^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScanner *wrapper = ImageScanner::Unwrap<ImageScanner>(info.This());

      try 
      {
        Platform::String^ result = wrapper->_instance->DeviceId;
        return scope.Close(NodeRT::Utils::NewString(result->Data()));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> FeederConfigurationGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScanner^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScanner *wrapper = ImageScanner::Unwrap<ImageScanner>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerFeederConfiguration^ result = wrapper->_instance->FeederConfiguration;
        return scope.Close(WrapImageScannerFeederConfiguration(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    
    static Handle<Value> FlatbedConfigurationGetter(Local<String> property, const AccessorInfo &info)
    {
      HandleScope scope;
      
      if (!NodeRT::Utils::IsWinRtWrapperOf<::Windows::Devices::Scanners::ImageScanner^>(info.This()))
      {
        return scope.Close(Undefined());
      }

      ImageScanner *wrapper = ImageScanner::Unwrap<ImageScanner>(info.This());

      try 
      {
        ::Windows::Devices::Scanners::ImageScannerFlatbedConfiguration^ result = wrapper->_instance->FlatbedConfiguration;
        return scope.Close(WrapImageScannerFlatbedConfiguration(result));
      }
      catch (Platform::Exception ^exception)
      {
        NodeRT::Utils::ThrowWinRtExceptionInJs(exception);
        return scope.Close(Undefined());
      }
    }
    


  private:
    ::Windows::Devices::Scanners::ImageScanner^ _instance;
    static Persistent<FunctionTemplate> s_constructorTemplate;

    friend v8::Handle<v8::Value> WrapImageScanner(::Windows::Devices::Scanners::ImageScanner^ wintRtInstance);
    friend ::Windows::Devices::Scanners::ImageScanner^ UnwrapImageScanner(Handle<Value> value);
    friend bool IsImageScannerWrapper(Handle<Value> value);
  };
  Persistent<FunctionTemplate> ImageScanner::s_constructorTemplate;

  v8::Handle<v8::Value> WrapImageScanner(::Windows::Devices::Scanners::ImageScanner^ winRtInstance)
  {
    HandleScope scope;

    if (winRtInstance == nullptr)
    {
      return scope.Close(Undefined());
    }

    Handle<Object> opaqueWrapper = CreateOpaqueWrapper(winRtInstance);
    Handle<Value> args[] = {opaqueWrapper};
    return scope.Close(ImageScanner::s_constructorTemplate->GetFunction()->NewInstance(_countof(args), args));
  }

  ::Windows::Devices::Scanners::ImageScanner^ UnwrapImageScanner(Handle<Value> value)
  {
     return ImageScanner::Unwrap<ImageScanner>(value.As<Object>())->_instance;
  }

  void InitImageScanner(Handle<Object> exports)
  {
    ImageScanner::Init(exports);
  }

} } } } 

void init(Handle<Object> exports)
{
  if (FAILED(CoInitializeEx(nullptr, COINIT_MULTITHREADED)))
  {
    ThrowException(v8::Exception::Error(NodeRT::Utils::NewString(L"error in CoInitializeEx()")));
    return;
  }
  
  NodeRT::Windows::Devices::Scanners::InitImageScannerFormatEnum(exports);
  NodeRT::Windows::Devices::Scanners::InitImageScannerAutoCroppingModeEnum(exports);
  NodeRT::Windows::Devices::Scanners::InitImageScannerColorModeEnum(exports);
  NodeRT::Windows::Devices::Scanners::InitImageScannerScanSourceEnum(exports);
  NodeRT::Windows::Devices::Scanners::InitIImageScannerFormatConfiguration(exports);
  NodeRT::Windows::Devices::Scanners::InitImageScannerAutoConfiguration(exports);
  NodeRT::Windows::Devices::Scanners::InitIImageScannerSourceConfiguration(exports);
  NodeRT::Windows::Devices::Scanners::InitImageScannerFlatbedConfiguration(exports);
  NodeRT::Windows::Devices::Scanners::InitImageScannerFeederConfiguration(exports);
  NodeRT::Windows::Devices::Scanners::InitImageScannerScanResult(exports);
  NodeRT::Windows::Devices::Scanners::InitImageScannerPreviewResult(exports);
  NodeRT::Windows::Devices::Scanners::InitImageScanner(exports);

  NodeRT::Utils::RegisterNameSpace("Windows.Devices.Scanners", exports);
}


NODE_MODULE(binding, init)